@@ -1,56 +1,58 @@
 import anndata
-import numpy as np
-from statsmodels.stats.multitest import multipletests
-import seaborn as sns
 import matplotlib.pyplot as plt
+import numpy as np
 import pandas as pd
+import seaborn as sns
+from statsmodels.stats.multitest import multipletests
 
 
 def _calculate_enrichment_score(raw_adata, labels):
     """
     Enrichment score modified from :cite:p:`Zeisel2018` for normalized methylation fractions
+    Assuming the methylation value is posterior frac calculated by MCDS.add_mc_frac)
     """
     n_cells = raw_adata.shape[0]
-    sizes = labels.value_counts()
+    total_hypo_n = pd.Series(np.ravel((raw_adata.X < 1).sum(axis=0)).copy(),
+                             index=raw_adata.var_names)
+    total_sum = pd.Series(np.ravel(raw_adata.X.sum(axis=0).copy()),
+                          index=raw_adata.var_names)
 
-    # hypo-methylated cells in the cluster
-    in_hypo_n = (
-        pd.DataFrame(
-            raw_adata.X < 1, index=raw_adata.obs_names, columns=raw_adata.var_names
-        )
-        .groupby(labels)
-        .sum()
-    )
-    in_hypo_frac = in_hypo_n / sizes.values[:, None]
-    # hypo-methylated cells not in the cluster
-    feature_sum = in_hypo_n.sum(axis=0)
-    out_hypo_n = feature_sum.values[None, :] - in_hypo_n
-    out_sizes = n_cells - sizes
-    out_hypo_frac = out_hypo_n / out_sizes.values[:, None]
+    enrichment = []
+    label_orders = []
+    for label, sub_df in raw_adata.obs.groupby(labels):
+        in_cells = sub_df.shape[0]
+        out_cells = n_cells - in_cells
+        if in_cells < 2:
+            continue
 
-    # mean of cells in the cluster
-    in_mean = (
-        pd.DataFrame(
-            raw_adata.X, index=raw_adata.obs_names, columns=raw_adata.var_names
-        )
-        .groupby(labels)
-        .mean()
-    )
-    # mean of cells not in the cluster
-    out_mean = (
-        raw_adata.X.sum(axis=0)[None, :] - in_mean * sizes.values[:, None]
-    ) / out_sizes.values[:, None]
+        label_orders.append(label)
+        # hypo-methylated cells in the cluster
+        in_hypo_n = pd.Series(np.ravel(
+            (raw_adata[sub_df.index, :].X < 1).sum(axis=0)).copy(),
+                              index=raw_adata.var_names)
+        in_hypo_frac = in_hypo_n / in_cells
+        # hypo-methylated cells not in the cluster
+        out_hypo_n = total_hypo_n - in_hypo_n
+        out_hypo_frac = out_hypo_n / out_cells
 
-    # finally, the enrichment score
-    # In Zeisel et al, the frac and mean change on the same direction,
-    # but in the mc frac case, larger fraction means smaller mean
-    # so the fraction part is reversed
-    enrichment = ((out_hypo_frac + 0.1) / (in_hypo_frac + 0.1)) * (
-        (in_mean + 0.01) / (out_mean + 0.01)
-    )
-    # enrichment direction is the same as mC fraction
-    # enrichment < 1, the gene is hypo-methylated in that cluster
-    # enrichment > 1, the gene is hyper-methylated in that cluster
+        # mean of cells in the cluster
+        in_mean = pd.Series(np.ravel(
+            raw_adata[sub_df.index, :].X.mean(axis=0)).copy(),
+                            index=raw_adata.var_names)
+        # mean of cells not in the cluster
+        out_mean = (total_sum - in_mean * in_cells) / out_cells
+
+        # finally, the enrichment score
+        # In Zeisel et al., the frac and mean change on the same direction,
+        # but in the mc frac case, larger fraction means smaller mean
+        # so the fraction part is reversed
+        e = ((in_hypo_frac + 0.1) / (out_hypo_frac + 0.1) *
+             (out_mean + 0.01) / (in_mean + 0.01))
+        enrichment.append(e)
+        # enrichment > 1, the gene is hypo-methylated in that cluster, the larger the better
+
+    enrichment = pd.DataFrame(enrichment,
+                              index=label_orders).loc[:, raw_adata.var_names].copy()
     return enrichment
 
 
@@ -60,41 +62,48 @@ def _calculate_enrichment_score_cytograph(adata, labels):
     """
     n_cells = adata.shape[0]
 
-    # Number of cells per cluster
-    sizes = labels.value_counts()
-
     clusters = []
     nnz = []
+    nnz_other = []
     means = []
+    means_other = []
+    sizes = []  # Number of cells per cluster
+    total_nnz = np.ravel((adata.X > 0).sum(axis=0))
+    total_sum = np.ravel(adata.X.sum(axis=0))
+    total_cells = adata.shape[0]
     for cluster, sub_df in adata.obs.groupby(labels):
         clusters.append(cluster)
-        # Number of nonzero values per cluster
-        nnz.append(np.ravel((adata[sub_df.index].X > 0).sum(axis=0)))
+        this_cells = sub_df.shape[0]
+        sizes.append(this_cells)
+        is_in_cluster = adata.obs_names.isin(sub_df.index)
+        # Number of nonzero values per cluster and other cells
+        this_nnz = np.ravel((adata[is_in_cluster, :].X > 0).sum(axis=0))
+        nnz.append(this_nnz)
+        other_nnz = total_nnz - this_nnz
+        nnz_other.append(other_nnz)
         # Mean value per cluster
-        means.append(np.ravel(adata[sub_df.index].X.mean(axis=0)))
-    nnz = np.vstack(nnz)
+        this_mean = np.ravel(adata[is_in_cluster, :].X.mean(axis=0))
+        means.append(this_mean)
+        other_mean = (total_sum - this_mean * this_cells) / \
+                     (total_cells - this_cells)
+        means_other.append(other_mean)
+    nnz = np.vstack(nnz).astype(np.float64)
+    nnz_other = np.vstack(nnz_other).astype(np.float64)
     means = np.vstack(means)
+    means_other = np.vstack(means_other)
+    sizes = np.array(sizes)
 
-    nnz_overall = nnz.sum(axis=0)
-    means_overall = np.ravel(adata.X.mean(axis=0))
+    # turn nnz count into fractions
+    _sizes = sizes[:, None]
+    _sizes_other = n_cells - _sizes
+    nnz /= _sizes
+    nnz_other /= _sizes_other
 
-    _sizes = sizes.values[:, None]
-    f_nnz = nnz / _sizes
-    f_nnz_overall = nnz_overall / n_cells
-    # Means and fraction non-zero values in other clusters (per cluster)
-    means_other = ((means_overall * n_cells)[None] - (means * _sizes)) / (
-        n_cells - _sizes
-    )
-    means_other[means_other < 0] = 0
-    f_nnz_other = ((f_nnz_overall * n_cells)[None] - (f_nnz * _sizes)) / (
-        n_cells - _sizes
-    )
-    f_nnz_other[f_nnz_other < 0] = 0
-
-    enrichment = (
-        (f_nnz + 0.1) / (f_nnz_other + 0.1) * (means + 0.01) / (means_other + 0.01)
-    )
-    enrichment = pd.DataFrame(enrichment, index=clusters, columns=adata.var_names)
+    enrichment = ((nnz + 0.1) / (nnz_other + 0.1) * (means + 0.01) /
+                  (means_other + 0.01))
+    enrichment = pd.DataFrame(enrichment,
+                              index=clusters,
+                              columns=adata.var_names)
     return enrichment
 
 
@@ -104,25 +113,28 @@ def _plot_enrichment_result(qvals, enrichment, null_enrichment, alpha):
 
     # plot feature with q_value pass cutoff
     ax = f_ax
-    sns.histplot((qvals < alpha).sum(axis=0), bins=30, ax=ax)
+    sns.histplot((qvals < alpha).sum(axis=0), ax=ax)
     ax.set_xlabel("# of clusters")
     ax.set_title(f"Features enriched in # of clusters (q<{alpha})")
     sns.despine(ax=ax)
 
     # plot q_value
     ax = q_ax
-    sns.histplot(qvals.ravel(), ax=ax, bins=50)
-    ax.set(yticks=[], ylabel="", xlabel="q value", title="q value distribution")
+    sns.histplot(np.ravel(qvals), ax=ax, bins=50)
+    ax.set(yticks=[],
+           ylabel="",
+           xlabel="q value",
+           title="q value distribution")
     sns.despine(ax=ax, left=True)
 
     # plot enrichment distribution
     ax = e_ax
-    plot_data = pd.DataFrame(
-        {
-            "enrichment": enrichment.unstack().reset_index(drop=True),
-            "null": null_enrichment.unstack().reset_index(drop=True),
-        }
-    )
+    plot_data = pd.DataFrame({
+        "enrichment":
+            enrichment.unstack().reset_index(drop=True),
+        "null":
+            null_enrichment.unstack().reset_index(drop=True),
+    })
     plot_data = plot_data.unstack().reset_index()
     plot_data.columns = ["Type", "_", "Enrichment Score"]
     sns.histplot(
@@ -145,7 +157,8 @@ def _aggregate_enrichment(adata, enrichment, top_n, alpha, qvals, cluster_col):
     # calculate use_features without qvals
     use_features_no_q = []
     for _, row in enrichment.iterrows():
-        use_features_no_q.append(row.sort_values(ascending=False)[:top_n].dropna())
+        use_features_no_q.append(
+            row.sort_values(ascending=False)[:top_n].dropna())
     use_features_no_q = pd.concat(use_features_no_q).index.unique()
 
     # mask non-sig enrichment scores, calculate again
@@ -156,23 +169,24 @@ def _aggregate_enrichment(adata, enrichment, top_n, alpha, qvals, cluster_col):
     use_features = pd.concat(use_features).index.unique()
     print(f"Selected {use_features.size} unique features")
     if len(use_features) == 0:
-        print(
-            f"No features found significantly enriched, "
-            f"use top enriched features that did not pass q<{alpha}"
-        )
+        print(f"No features found significantly enriched, "
+              f"use top enriched features that did not pass q<{alpha}")
         use_features = use_features_no_q
 
     adata.uns[f"{cluster_col}_feature_enrichment"] = {
-        "qvals": qvals.T,
+        "qvals": np.array(qvals).T,
+        "enrichment": np.array(enrichment).T,
         "cluster_order": enrichment.index.tolist(),
     }
     return use_features
 
 
-def cluster_enriched_features(
-    adata: anndata.AnnData, cluster_col: str,
-    top_n=200, alpha=0.05, stat_plot=True, method="mc"
-):
+def cluster_enriched_features(adata: anndata.AnnData,
+                              cluster_col: str,
+                              top_n=200,
+                              alpha=0.05,
+                              stat_plot=True,
+                              method="mc"):
     """
     Calculate top Cluster Enriched Features (CEF) from per-cell normalized dataset.
     An post-clustering feature selection step adapted from :cite:p:`Zeisel2018,La_Manno2021`
@@ -230,18 +244,24 @@ def cluster_enriched_features(
     else:
         print("Computing enrichment score")
         enrichment = use_func(adata, adata.obs[cluster_col])
-        null_label = adata.obs.sample(n=adata.shape[0])[cluster_col]
+        # shuffle cluster label to calculate null dist
+        null_label = adata.obs[cluster_col].sample(n=adata.shape[0],
+                                                   replace=False)
         null_label.index = adata.obs_names
         null_enrichment = use_func(adata, null_label)
 
         print("Computing enrichment score FDR-corrected P values")
+        # FDR correction per row, row is cluster
         qvals = np.zeros_like(enrichment)
-        for ix in range(enrichment.shape[1]):
-            null_values = null_enrichment.iloc[:, ix].sort_values()
-            values = enrichment.iloc[:, ix]
+        for row in range(enrichment.shape[0]):
+            null_values = null_enrichment.iloc[row, :].sort_values()
+            values = enrichment.iloc[row, :]
             pvals = 1 - np.searchsorted(null_values, values) / values.shape[0]
             (_, q, _, _) = multipletests(pvals, alpha, method="fdr_bh")
-            qvals[:, ix] = q
+            qvals[row, :] = q
+        qvals = pd.DataFrame(qvals,
+                             index=enrichment.index,
+                             columns=enrichment.columns)
 
         if stat_plot:
             _plot_enrichment_result(
@@ -261,5 +281,6 @@ def cluster_enriched_features(
         )
 
     # save the calculated results in the input adata
-    adata.var[f"{cluster_col}_enriched_features"] = adata.var_names.isin(use_features)
+    adata.var[f"{cluster_col}_enriched_features"] = adata.var_names.isin(
+        use_features)
     return
